# Package:  app
# Date:     20th June 2009
# Author:   James Mills, prologic at shortcircuit dot net dot au

"""Application Components

Contains various components useful for application development and tasks
common to applications.

:copyright: CopyRight (C) 2004-2013 by James Mills
:license: MIT (See: LICENSE)
"""

from .daemon import Daemon

__all__ = ("Daemon",)

# flake8: noqa
